using System;
using System.Collections.Generic;
using System.Text;
using OSGeo.FDO.Connections;
using OSGeo.FDO.ClientServices;
using OSGeo.FDO.Commands.DataStore;
using OSGeo.FDO.Schema;
using OSGeo.FDO.Commands.Schema;
using OSGeo.FDO.Commands.Feature;
using OSGeo.FDO.Commands;
using OSGeo.FDO.Expression;

namespace Test
{
    class Program
    {
        static void Main(string[] args)
        {
            // Expected output:
            //
            // id|a|b|c|d|e|
            // 
            // 1|Foo|<null>|<null>|<null>|Bar|
            // 2|<null>|Bar|<null>|<null>|Foo|
            // 3|<null>|<null>|Bar|Snafu|<null>|
            // 4|<null>|<null>|<null>|<null>|Whatever|
            //
            // Actual result (with SQLite, SDF produces the desired result):
            //
            // id|a|b|c|d|e|
            //
            // 1|Foo|<null>|<null>|<null>|Bar|
            // 2|Bar|<null>|<null>|<null>|Foo|
            // 3|Bar|<null>|<null>|<null>|Snafu|
            // 4|Whatever|<null>|<null>|<null>|Snafu|

            string provider = "OSGeo.SQLite"; //"OSGeo.SDF";
            string file = "test.db"; //"test.sdf";

            if (System.IO.File.Exists(file))
                System.IO.File.Delete(file);

            IConnection conn = FeatureAccessManager.GetConnectionManager().CreateConnection(provider);
            using(ICreateDataStore create = (ICreateDataStore)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_CreateDataStore))
            {
                create.DataStoreProperties.SetProperty("File", file);
                create.Execute();
            }
            conn.ConnectionString = "File=" + file;
            conn.Open();

            //Create our test class
            Class cls = new Class("Test", "");

            DataPropertyDefinition id = new DataPropertyDefinition("id", "");
            id.DataType = DataType.DataType_Int32;
            id.IsAutoGenerated = true;
            id.ReadOnly = true;

            cls.Properties.Add(id);
            cls.IdentityProperties.Add(id);

            DataPropertyDefinition a = new DataPropertyDefinition("a", "");
            a.DataType = DataType.DataType_String;
            a.Length = 255;
            a.Nullable = true;

            cls.Properties.Add(a);

            DataPropertyDefinition b = new DataPropertyDefinition("b", "");
            b.DataType = DataType.DataType_String;
            b.Length = 255;
            b.Nullable = true;

            cls.Properties.Add(b);

            DataPropertyDefinition c = new DataPropertyDefinition("c", "");
            c.DataType = DataType.DataType_String;
            c.Length = 255;
            c.Nullable = true;

            cls.Properties.Add(c);


            DataPropertyDefinition d = new DataPropertyDefinition("d", "");
            d.DataType = DataType.DataType_String;
            d.Length = 255;
            d.Nullable = true;

            cls.Properties.Add(d);


            DataPropertyDefinition e = new DataPropertyDefinition("e", "");
            e.DataType = DataType.DataType_String;
            e.Length = 255;
            e.Nullable = true;

            cls.Properties.Add(e);

            FeatureSchema schema = new FeatureSchema("Default", "");
            schema.Classes.Add(cls);

            using (IApplySchema apply = (IApplySchema)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_ApplySchema))
            {
                apply.FeatureSchema = schema;
                apply.Execute();
            }

            //Insert 4 rows of data

            IInsert insert = (IInsert)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_Insert);
            insert.SetFeatureClassName("Test");
            PropertyValueCollection propVals = insert.PropertyValues;

            // a = "Foo", e = "Bar"
            propVals.Add(new PropertyValue("a", new StringValue("Foo")));
            propVals.Add(new PropertyValue("e", new StringValue("Bar")));

            using (IFeatureReader reader = insert.Execute())
            {
                reader.Close();
            }

            // b = "Bar", e = "Foo"
            propVals.Clear();
            propVals.Add(new PropertyValue("b", new StringValue("Bar")));
            propVals.Add(new PropertyValue("e", new StringValue("Foo")));

            using (IFeatureReader reader = insert.Execute())
            {
                reader.Close();
            }

            // c = "Bar", d = "Snafu"
            propVals.Clear();
            propVals.Add(new PropertyValue("c", new StringValue("Bar")));
            propVals.Add(new PropertyValue("d", new StringValue("Snafu")));

            using (IFeatureReader reader = insert.Execute())
            {
                reader.Close();
            }

            // e = "Whatever"
            propVals.Clear();
            propVals.Add(new PropertyValue("e", new StringValue("Whatever")));

            using (IFeatureReader reader = insert.Execute())
            {
                reader.Close();
            }

            //Now print the rows

            using (ISelect sel = (ISelect)conn.CreateCommand(CommandType.CommandType_Select))
            {
                sel.SetFeatureClassName("Test");

                using (IFeatureReader reader = sel.Execute())
                {
                    ClassDefinition cd = reader.GetClassDefinition();
                    foreach (PropertyDefinition pd in cd.Properties)
                    {
                        Console.Write(pd.Name + "|");
                    }
                    Console.WriteLine("\n");
                    while (reader.ReadNext())
                    {
                        foreach (PropertyDefinition pd in cd.Properties)
                        {
                            if (!reader.IsNull(pd.Name))
                            {
                                if (pd.PropertyType == PropertyType.PropertyType_DataProperty)
                                {
                                    DataPropertyDefinition dp = pd as DataPropertyDefinition;
                                    switch (dp.DataType)
                                    {
                                        case DataType.DataType_String:
                                            Console.Write(reader.GetString(dp.Name));
                                            Console.Write("|");
                                            break;
                                        case DataType.DataType_Int32:
                                            Console.Write(reader.GetInt32(dp.Name));
                                            Console.Write("|");
                                            break;
                                    }
                                }
                            }
                            else
                            {
                                Console.Write("<null>|");
                            }
                        }
                        Console.WriteLine();
                    }
                }
            }

            conn.Close();
            conn.Dispose();

            Console.Read();
        }
    }
}
